#
# Copyright (c) 2013-2023 NIBIO.
#
# This file is part of VIPSWeb 
# (see https://gitlab.nibio.no/VIPS/VIPSWeb).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
#

from security.models import VipsLogicUser
from django.conf import settings
from datetime import datetime, timedelta
import json
from django.http import HttpResponse


class CheckLoginMiddleware(object):
    datetime_format = "%Y-%m-%d %H:%M:%S"
    
    def __init__(self, get_response):
        self.get_response = get_response
        
    def __call__(self, request):
        self.process_request(request)
        return self.get_response(request)
    
    
    def process_request(self, request):
        # If UUID is provided, login with VIPSLogic
        # VIPSLogicUser exists in session for 24 hours
        # UUID exists for 30 days by default
        return_uuid = request.GET.get("returnUUID",None)
        if return_uuid != None:
            found_user = VipsLogicUser.find_by_uuid(return_uuid)
            if found_user != None:
                request.session["vips_logic_user"] = found_user
                request.session["user_uuid"] = return_uuid
                request.session["last_modified"] = datetime.now().strftime(CheckLoginMiddleware.datetime_format)
                request.session.set_expiry(2592000) # 30 days lasting
            else:
                    request.session["vips_logic_user"] = None
                    request.session["user_uuid"] = None
                    request.session["last_modified"] = None
                    request.session.set_expiry(0)
        else:
            # If not, check if current login is valid
            # 1. Is last_modified not present or more than 24 hours ago?
            # If so: check with VIPSLogic if user_uuid is valid
            last_modified = request.session.get("last_modified", None)
            #print last_modified
            age_limit = timedelta(days = 1)
            #age_limit = timedelta(seconds = 10)
            if  last_modified == None or datetime.strptime(last_modified, CheckLoginMiddleware.datetime_format) < datetime.now() - age_limit:
                # 2. If not, check if UUID is present
                # If UUID present, try to login to VIPSLogic with it
                user_uuid = request.session.get("user_uuid", None)
                if user_uuid != None:
                    found_user = VipsLogicUser.find_by_uuid(user_uuid)
                    if found_user != None:
                        #print ("Found user. Logging in.")
                        request.session["vips_logic_user"] = found_user
                        request.session["user_uuid"] = user_uuid
                        request.session["last_modified"] = datetime.now().strftime(CheckLoginMiddleware.datetime_format)
                        request.session.set_expiry(2592000) # 30 days lasting
                    else:
                        request.session["vips_logic_user"] = None
                        request.session["user_uuid"] = None
                        request.session["last_modified"] = None
                        request.session.set_expiry(0)